@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: stage.py 1145 2007-02-24 17:30:34Z pronovic $
+# Revision : $Id: stage.py 1181 2007-03-25 16:18:22Z pronovic $
 # Purpose  : Implements the standard 'stage' action.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -101,7 +101,7 @@ def executeStage(configPath, options, config):
    @raise ValueError: Under many generic error conditions
    @raise IOError: If there are problems reading or writing files.
    """
-   logger.debug("Executing stage action.")
+   logger.debug("Executing the 'stage' action.")
    if config.options is None or config.stage is None:
       raise ValueError("Stage configuration is not properly filled in.")
    dailyDir = _getDailyDir(config)
