@@ -11,7 +11,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: release.py 1158 2007-02-27 02:51:42Z pronovic $
+# Revision : $Id: release.py 1191 2007-03-26 01:11:08Z pronovic $
 # Purpose  : Provides location to maintain release information.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -34,7 +34,7 @@ Provides location to maintain version information.
 AUTHOR      = "Kenneth J. Pronovici"
 EMAIL       = "pronovic@ieee.org"
 COPYRIGHT   = "2004-2007"
-VERSION     = "2.12.1"
-DATE        = "26 Feb 2007"
+VERSION     = "2.13.0"
+DATE        = "25 Mar 2007"
 URL         = "http://cedar-solutions.com/software/cedar-backup"
 
